@@ -13,6 +13,7 @@ sub finish_job   { croak 'Method "finish_job" not implemented by subclass' }
 sub job_info     { croak 'Method "job_info" not implemented by subclass' }
 sub list_jobs    { croak 'Method "list_jobs" not implemented by subclass' }
 sub list_workers { croak 'Method "list_workers" not implemented by subclass' }
+sub lock         { croak 'Method "lock" not implemented by subclass' }
 sub receive      { croak 'Method "receive" not implemented by subclass' }
 
 sub register_worker {
@@ -24,6 +25,7 @@ sub repair     { croak 'Method "repair" not implemented by subclass' }
 sub reset      { croak 'Method "reset" not implemented by subclass' }
 sub retry_job  { croak 'Method "retry_job" not implemented by subclass' }
 sub stats      { croak 'Method "stats" not implemented by subclass' }
+sub unlock     { croak 'Method "unlock" not implemented by subclass' }
 
 sub unregister_worker {
   croak 'Method "unregister_worker" not implemented by subclass';
@@ -52,6 +54,7 @@ Minion::Backend - Backend base class
   sub job_info          {...}
   sub list_jobs         {...}
   sub list_workers      {...}
+  sub lock              {...}
   sub receive           {...}
   sub register_worker   {...}
   sub remove_job        {...}
@@ -59,6 +62,7 @@ Minion::Backend - Backend base class
   sub reset             {...}
   sub retry_job         {...}
   sub stats             {...}
+  sub unlock            {...}
   sub unregister_worker {...}
   sub worker_info       {...}
 
@@ -363,6 +367,27 @@ List only jobs for this task.
 Returns the same information as L</"worker_info"> but in batches. Meant to be
 overloaded in a subclass.
 
+=head2 lock
+
+  my $bool = $backend->lock('foo', 3600);
+  my $bool = $backend->lock('foo', 3600, {limit => 20});
+
+Try to acquire a named lock that will expire automatically after the given
+amount of time in seconds.
+
+These options are currently available:
+
+=over 2
+
+=item limit
+
+  limit => 20
+
+Number of shared locks with the same name that can be active at the same time,
+defaults to C<1>.
+
+=back
+
 =head2 receive
 
   my $commands = $backend->receive($worker_id);
@@ -499,6 +524,12 @@ Number of workers that are currently not processing a job.
 
 =back
 
+=head2 unlock
+
+  my $bool = $backend->unlock('foo');
+
+Release a named lock.
+
 =head2 unregister_worker
 
   $backend->unregister_worker($worker_id);
