@@ -1,8 +1,7 @@
 package Minion::Command::minion::worker;
 use Mojo::Base 'Mojolicious::Command';
 
-use Getopt::Long qw(GetOptionsFromArray :config no_auto_abbrev no_ignore_case);
-use Mojo::Util 'steady_time';
+use Mojo::Util qw(getopt steady_time);
 
 has description => 'Start Minion worker';
 has usage => sub { shift->extract_usage };
@@ -10,15 +9,21 @@ has usage => sub { shift->extract_usage };
 sub run {
   my ($self, @args) = @_;
 
-  GetOptionsFromArray \@args,
-    'C|command-interval=i'   => \($self->{commands}  = 10),
+  my $app    = $self->app;
+  my $worker = $self->{worker} = $app->minion->worker;
+  my $status = $worker->status;
+  $status->{performed} //= 0;
+
+  getopt \@args,
+    'C|command-interval=i'   => \($status->{command_interval}   //= 10),
     'f|fast-start'           => \my $fast,
-    'I|heartbeat-interval=i' => \($self->{heartbeat} = 60),
-    'j|jobs=i'               => \($self->{max}       = 4),
-    'q|queue=s'              => \my @queues,
-    'R|repair-interval=i'    => \($self->{repair}    = 21600);
-  $self->{queues} = @queues ? \@queues : ['default'];
-  $self->_next_repair if $fast;
+    'I|heartbeat-interval=i' => \($status->{heartbeat_interval} //= 300),
+    'j|jobs=i'               => \($status->{jobs}               //= 4),
+    'q|queue=s'              => ($status->{queues}              //= []),
+    'R|repair-interval=i'    => \($status->{repair_interval}    //= 21600);
+  @{$status->{queues}} = ('default') unless @{$status->{queues}};
+  $status->{repair_interval} -= int rand $status->{repair_interval} / 2;
+  $self->{last_repair} = $fast ? steady_time : 0;
 
   local $SIG{CHLD} = 'DEFAULT';
   local $SIG{INT} = local $SIG{TERM} = sub { $self->{finished}++ };
@@ -26,10 +31,8 @@ sub run {
     = sub { ++$self->{finished} and kill 'KILL', keys %{$self->{jobs}} };
 
   # Remote control commands need to validate arguments carefully
-  my $app = $self->app;
-  my $worker = $self->{worker} = $app->minion->worker;
   $worker->add_command(
-    jobs => sub { $self->{max} = $_[1] if ($_[1] // '') =~ /^\d+$/ });
+    jobs => sub { $status->{jobs} = $_[1] if ($_[1] // '') =~ /^\d+$/ });
   $worker->add_command(
     stop => sub { $self->{jobs}{$_[1]}->stop if $self->{jobs}{$_[1] // ''} });
 
@@ -40,41 +43,39 @@ sub run {
   $worker->unregister;
 }
 
-sub _next_repair {
-  my $self = shift;
-  $self->{check}
-    = steady_time + ($self->{repair} - int rand $self->{repair} / 2);
-}
-
 sub _work {
   my $self = shift;
 
   # Send heartbeats in regular intervals
   my $worker = $self->{worker};
-  $worker->register and $self->{lr} = steady_time + $self->{heartbeat}
-    if ($self->{lr} || 0) < steady_time;
+  my $status = $worker->status;
+  $self->{last_heartbeat} ||= 0;
+  $worker->register and $self->{last_heartbeat} = steady_time
+    if ($self->{last_heartbeat} + $status->{heartbeat_interval}) < steady_time;
 
   # Process worker remote control commands in regular intervals
-  $worker->process_commands and $self->{lp} = steady_time + $self->{commands}
-    if ($self->{lp} || 0) < steady_time;
+  $self->{last_command} ||= 0;
+  $worker->process_commands and $self->{last_command} = steady_time
+    if ($self->{last_command} + $status->{command_interval}) < steady_time;
 
   # Repair in regular intervals (randomize to avoid congestion)
-  if (($self->{check} || 0) < steady_time) {
+  if (($self->{last_repair} + $status->{repair_interval}) < steady_time) {
     my $app = $self->app;
     $app->log->debug('Checking worker registry and job queue');
     $app->minion->repair;
-    $self->_next_repair;
+    $self->{last_repair} = steady_time;
   }
 
   # Check if jobs are finished
   my $jobs = $self->{jobs} ||= {};
-  $jobs->{$_}->is_finished and delete $jobs->{$_} for keys %$jobs;
+  $jobs->{$_}->is_finished and ++$status->{performed} and delete $jobs->{$_}
+    for keys %$jobs;
 
   # Wait if job limit has been reached or worker is stopping
-  if (($self->{max} <= keys %$jobs) || $self->{finished}) { sleep 1 }
+  if (($status->{jobs} <= keys %$jobs) || $self->{finished}) { sleep 1 }
 
   # Try to get more jobs
-  elsif (my $job = $worker->dequeue(5 => {queues => $self->{queues}})) {
+  elsif (my $job = $worker->dequeue(5 => {queues => $status->{queues}})) {
     $jobs->{my $id = $job->id} = $job->start;
     my ($pid, $task) = ($job->pid, $job->task);
     $self->app->log->debug(
@@ -108,7 +109,7 @@ Minion::Command::minion::worker - Minion worker command
         --home <path>                    Path to home directory of your
                                          application, defaults to the value of
                                          MOJO_HOME or auto-detection
-    -I, --heartbeat-interval <seconds>   Heartbeat interval, defaults to 60
+    -I, --heartbeat-interval <seconds>   Heartbeat interval, defaults to 300
     -j, --jobs <number>                  Maximum number of jobs to perform
                                          parallel in forked worker processes,
                                          defaults to 4
